@@ -1,20 +1,16 @@
 package Reddit::Client;
 
-our $VERSION = '1.27';
+our $VERSION = '1.28';
+# TODO: make ispost, iscomment and get_type static
+# 1.28  -get_links_by_id
+#		-get_collapsed_comments
+#		-get_comments new version
+#		-MoreComments objects
+#		-added function get_replies to Link
+#       -can change subdomain (www to old or new, etc)
+#       -removed line number message on die. should this be an option instead?
+#		-get_permalink renamed ro get_web_url in Link and Comment
 # 1.27  print_request_on_error
-# 1.27  added alias function get_modqueue
-# 1.26  crossposts
-# 1.252 Dropped token refresh time to 3550 (1 hour - 50 seconds)
-# 1.251 removed unnecessary passing in of variables to get_token in request
-# 1.25	username can now optionally be passed in for web type apps
-# 1.244 token request is called in static context; if failed it would look for $self->{request_errors} but there was no self
-# 1.243 rolled back change making request_errors default
-# 1.242 request_errors (print the content of errors) is now true by default.
-# 1.24 ban function
-# 1.241 fix newline in request->send
-# be pretty amazed if we didn't break anything 
-# 1.23 get_modmail_raw
-#	   get_refresh_token
 
 $VERSION = eval $VERSION;
 
@@ -35,6 +31,7 @@ require Reddit::Client::Link;
 require Reddit::Client::SubReddit;
 require Reddit::Client::Request;
 require Reddit::Client::Message;
+require Reddit::Client::MoreComments;
 
 #===============================================================================
 # Constants
@@ -123,6 +120,8 @@ use constant API_IGNORE_REPORTS      => 36;
 use constant API_GETWIKI             => 37;
 use constant API_GET_MODMAIL         => 38;
 use constant API_BAN                 => 39;
+use constant API_MORECHILDREN        => 40;
+use constant API_BY_ID               => 41;
 
 #===============================================================================
 # Parameters
@@ -131,8 +130,8 @@ use constant API_BAN                 => 39;
 our $DEBUG            = 0;
 our $BASE_URL         = 'https://oauth.reddit.com';
 use constant BASE_URL =>'https://oauth.reddit.com';
-our $LINK_URL         = 'https://www.reddit.com';
-use constant LINK_URL =>'https://www.reddit.com';
+our $LINK_URL         = 'https://www.reddit.com'; # Why are there two of these?
+use constant LINK_URL =>'https://www.reddit.com'; # both are unused now?
 our $UA               = sprintf 'Reddit::Client/%f', $VERSION;
 
 our @API;
@@ -176,6 +175,8 @@ $API[API_APPROVE       ] = ['POST', '/api/approve'            ];
 $API[API_IGNORE_REPORTS] = ['POST', '/api/ignore_reports'     ];
 $API[API_GET_MODMAIL   ] = ['GET',  '/api/mod/conversations'  ];
 $API[API_BAN           ] = ['POST', '/r/%s/api/friend'        ];
+$API[API_MORECHILDREN  ] = ['GET',  '/api/morechildren'       ];
+$API[API_BY_ID         ] = ['GET',  '/by_id'                  ];
 
 #===============================================================================
 # Class methods
@@ -193,8 +194,8 @@ use fields (
     'secret',       # always required
 	'username',		# now optional for web apps
 	'password',		# script apps only
-	'request_errors',		# print request errors
-	'print_request_errors',	# print request errors, deprecated
+	'request_errors',		# print request errors, deprecated
+	'print_request_errors',	# print request errors
 	'print_response',		# print response content, deprecated
 	'print_response_content',# print response content
 	'print_request', 		# print entire request
@@ -202,6 +203,7 @@ use fields (
 	'refresh_token', 		# oauth refresh token
 	'auth_type',			# 'script' or 'webapp'
 	'debug',
+	'subdomain',
 );
 
 sub new {
@@ -217,6 +219,7 @@ sub new {
 	$self->{print_request}  = $param{print_request} || 0;
 	$self->{debug}			= $param{debug}	|| 0;
 	$self->{print_request_on_error} = $param{print_request_on_error} || 0;
+	$self->{subdomain}		= $param{subdomain} || 'www';
 
 	if ($param{password}) {
 		if (!$param{username}) {
@@ -379,7 +382,7 @@ sub json_request {
 
     if ($method eq 'POST') {
         $post_data ||= {};
-        $post_data->{api_type} = 'json';
+        $post_data->{api_type} = 'json'; # only POST enpoints require*
     } else { 
         #$path .= '.json'; # the oauth api returns json by default
     }
@@ -557,7 +560,8 @@ sub mark_read {
 sub mark_inbox_read {
 	my $self = shift;
 	my ($method, $path) = @{$API[API_MARKALL]};
-	my $post_data = {api_type=>'json'};
+	# Why does this error without api_type? json_request is adding it anyway?
+	my $post_data = {api_type => 'json'};
 	my $result = $self->request($method, $path, {}, $post_data);
 }
 
@@ -583,8 +587,10 @@ sub info {
     my $query->{id} = $id;
     
     my $info = $self->api_json_request(
-	api => API_INFO, 
-	data=>$query);
+		api => API_INFO, 
+		data=>$query
+	);
+	#return $info;
     my $rtn = $info->{data}->{children}[0]->{data};
     $rtn->{kind} = $info->{data}->{children}[0]->{kind} if $rtn;
     return $rtn;
@@ -673,27 +679,46 @@ sub get_links { # alias for fetch_links to make naming convention consistent
     my ($self, %param) = @_;
 	return $self->fetch_links(%param);
 }
+# Is this a better way to get a single link than a call to info?
+sub get_links_by_id {
+	my ($self, @fullnames) = @_;
+	die "get_links_by_id: argument 1 (\@fullnames) is required.\n" unless @fullnames;
+	@fullnames = map { fullname($_, 't3') } @fullnames;
+	my $str = join ",", @fullnames;	
+	#my $result = $self->api_json_request(
+	$self->{print_request_on_error} = 1;
+	my $result = $self->json_request('GET', $API[API_BY_ID][1]."/$str");
+
+	return [
+		map { Reddit::Client::Link->new($self, $_->{data}) } @{$result->{data}{children}} 
+	];
+}
 
 sub get_link {
     	my ($self, $fullname) = @_;
 	croak "expected argument 1: id or fullname" unless $fullname;
 
-	$fullname = "t3_$fullname" if $self->get_type($fullname) ne "t3";
+	$fullname = fullname($fullname, 't3');
 	my $info = $self->info($fullname);
+	return unless $info;
 
-	return $info if !$info;
 	return Reddit::Client::Link->new($self, $info);
 }
 
 sub get_comment {
-    	my ($self, $fullname) = @_;
+    my ($self, $fullname, %param) = @_;
 	croak "expected argument 1: id or fullname" unless $fullname;
 
-	$fullname = "t1_$fullname" if $self->get_type($fullname) ne "t1";
+	$fullname = fullname($fullname, 't1');
 	my $info = $self->info($fullname);
+	return unless $info;
 
-	return $info if !$info;
-	return Reddit::Client::Comment->new($self, $info);
+	my $cmt = Reddit::Client::Comment->new($self, $info);
+	if ($param{include_children} and $cmt->{permalink}) {
+		$cmt = $self->get_comments(permalink=>$cmt->{permalink});
+		$cmt = $$cmt[0];
+	} 
+	return $cmt;
 }
 
 sub get_subreddit_comments {
@@ -772,7 +797,7 @@ sub ban {
 	my ($self, %param) = @_;
 	my $sub	= $param{sub} || $param{subreddit} || die "subreddit is required\n";
 	
-	my $data = { api_type => 'json' };
+	my $data = {}; 
 	$data->{name}	= $param{username} || die "username is required\n";
 	# ban_context = fullname, but of what - not required
 
@@ -817,9 +842,8 @@ sub ban {
 	);
 	return $result;
 }
-# modqueue - why was it not called that?
 sub get_modlinks {
-    	my ($self, %param) = @_;
+    my ($self, %param) = @_;
 
 	my $query = $self->set_listing_defaults(%param);
 	my $sub   = $param{sub} || $param{subreddit} || 'mod';
@@ -844,8 +868,9 @@ sub get_modlinks {
 		@{$result->{data}{children}} 
 	];
 }
-sub get_modqueue { # alias for get_modlinks
-	my ($self, %param) = @_;
+sub get_modqueue { 
+    my ($self, %param) = @_;
+	$param{mode} = 'modqueue';
 	return $self->get_modlinks(%param);
 }
 
@@ -853,14 +878,13 @@ sub get_modqueue { # alias for get_modlinks
 # entity: comma-delimited list of subreddit names
 # limit
 # sort: one of (recent, mod, user, unread)
-# state: one of (new, inprogress, mod, notifications, archived, highlighted, all)
+# state: one of (new, inprogress, mod, notifications, archived, highlighted, all
 sub get_modmail {
     	my ($self, %param) = @_;
 
 	my $data	= {};
 	$data->{sort}	= $param{sort} || 'unread';
 	$data->{state}	= $param{state} || 'all';
-	# What is the difference between sort 'unread' and state 'new'? Possibly state 'new' is unseen by any mod?
 	$data->{after}	= $param{after} if $param{after};
 	$data->{limit}	= exists $param{limit} ? ( $param{limit} ? $param{limit} : 500 )  : DEFAULT_LIMIT;
 
@@ -1188,15 +1212,76 @@ sub edit_wiki {
 #===============================================================================
 # Comments
 #===============================================================================
-
-sub get_comments { # currently broken
+sub get_comments { 
     my ($self, %param) = @_;
-    my $permalink = $param{permalink} || croak 'Expected "permalink"';
+    my $permalink;
+	my $sub		= $param{sub} || $param{subreddit};
 
-    DEBUG('Retrieve comments for %s', $permalink);
+	if ($param{permalink}) {
+		$permalink = $param{permalink};
+	} elsif ($sub and $param{comment_id} and $param{link_id}) {
+		my $id = id($param{link_id});
+		my $cmtid = id($param{comment_id});
+		$permalink = "/r/$sub/comments/$id//$cmtid";
+	} elsif ($sub and $param{id}) {
+		my $id = id($param{id});
+		$permalink = "/r/$sub/comments/$id";
+	} elsif ($param{url}) {
+		$permalink = $param{url};
+		$permalink =~ s/^https?:\/\/([a-zA-Z]{1,3}\.)?reddit\.com//i;
+	} else {
+		die "get_comments: Either 'permalink' OR 'url' OR 'subreddit' and 'link_id' OR 'subreddit' and 'link_id' and 'comment_id' are required.\n";
+	}
 
-    my $result   = $self->json_request('GET', $permalink);
+    my $result  = $self->json_request('GET', $permalink);
+	my $link_id	= $result->[0]{data}{children}[0]{data}{name};
+	# result->[0] is a listing with 1 element, the link, even if you requested a cmt
     my $comments = $result->[1]{data}{children};
+	#return $result;
+
+	my $return = [];
+	for my $cmt (@$comments) {
+		if ($cmt->{kind} eq 't1') {
+			push @$return, Reddit::Client::Comment->new($self, $cmt->{data});
+		} elsif ($cmt->{kind} eq 'more') {
+			my $more = Reddit::Client::MoreComments->new($self, $cmt->{data});
+			$more->{link_id} = $link_id;
+			push @$return, $more;
+		}
+	}
+	return $return;
+}
+# limit_children: get these comments and their descendants
+sub get_collapsed_comments {
+    my ($self, %param) = @_;
+	my $link_id		= fullname($param{link_id},'t3') || die "load_more_comments: 'link_id' is required.\n";
+	my $children	= $param{children} || die "get_collapsed_comments: 'children' is required.\n";
+	my $limit		= exists $param{limit_children} ? ($param{limit_children} ? 'true' : 'false') : 'false';
+	my $ids;
+
+	if (ref $children eq 'ARRAY') { 
+		$ids = join ",", @$children;
+		die "'children' must be non-empty array reference" unless $ids;
+	} else {
+		die "get_collapsed_comments: 'children' must be array reference\n"; 
+	}
+
+	my $data = {
+		link_id			=> $link_id,
+		children		=> $ids,	
+		limit_children	=> $limit, 
+		api_type		=> 'json', # This is the only GET endpoint that requires
+	};							   # api_type=json to be set.
+
+	$data->{sort} 		= $param{sort} if $param{sort};
+	$data->{id}			= $param{id} if $param{id};
+
+	my $result = $self->api_json_request(
+		api		=> API_MORECHILDREN,
+		data	=> $data,
+	);
+	#return $result;
+	my $comments = $result->{data}->{things};
     return [ map { Reddit::Client::Comment->new($self, $_->{data}) } @$comments ];
 }
 
@@ -1396,23 +1481,44 @@ sub delete_multi {
 	);
 	return $result->{data};
 }
+#==============================================================================
+# Misc
+#==============================================================================
+sub get_origin { 
+	my $self = shift;
+	return "https://$self->{subdomain}.reddit.com";	
+}
 
 #==============================================================================
-# Internal
+# Internal and static
 #==============================================================================
 
-sub ispost {
+# Strip the type portion of a filname (i.e. t3_), if it exists
+sub id {
+	my $id	= shift;
+	$id 	=~ s/^[tT]\d_//;
+	return $id;
+}
+# accept id or fullname, always return fullname
+sub fullname {
+	my $id 		= shift || return;
+	my $type	= shift || die "fullname: 'type' is required";
+	$id = $type."_".$id if substr($id, 0, 3) ne $type."_";
+	return $id;
+}
+
+sub ispost { # todo: make this static function
 	my ($self, $name) = @_;
     	my $type = substr $name, 0, 2;
 	return $type eq 't3';
 }
 
-sub iscomment {
+sub iscomment { # todo: make this static
 	my ($self, $name) = @_;
     	my $type = substr $name, 0, 2;
 	return $type eq 't1';
 }
-sub get_type {
+sub get_type { # ditto
 	my ($self, $name) = @_;
     	return lc substr $name, 0, 2;
 }
@@ -1525,14 +1631,13 @@ Up-to-date, nicely-formatted documentation can be found at L<http://redditclient
 
 =head1 DESCRIPTION
 
-Reddit::Client provides methods and object wrappers for objects exposed
-. This module handles HTTP communication, oauth session management, and communication with Reddit's external API. For more information about the Reddit API, see L<https://github.com/reddit/reddit/wiki/API>.
+Reddit::Client handles HTTP communication, oauth session management, and communication with Reddit's external API. For more information about the Reddit API, see L<https://github.com/reddit/reddit/wiki/API>.
 
-Beginning August 3rd, 2015, the Reddit API requires Oauth authentication. This amounts to two extra arguments at the beginning of your script; in exchange you you twice as many requests per minute as before (60 vs 30) and some added convenience on the back end.
+Beginning August 3rd, 2015, the Reddit API requires Oauth2 authentication. This amounts to two extra arguments at the beginning of your script (for basic authentication); in exchange you get twice as many requests per minute as before (60 vs 30) and some added convenience on the back end.
 
-To get Oauth keys, visit your apps page: L<https://www.reddit.com/prefs/apps>. Choose a "script" type app. None of the other fields are needed for a script type app; you can put in C<127.0.0.1> for the URLs or whatever you like.
+To get Oauth keys, visit your apps page: L<https://www.reddit.com/prefs/apps>. Choose a "script" type app. None of the other fields are needed for this type of app; the URL fields must be filled, but can be any valid URL.
 
-Every account that uses this script must have permission to use it. So for example if you were to check the inboxes of your various accounts, each one would need permission. To add accounts click "add developer" on the right, after you've created the app.
+As of v1.20, Reddit::Client supports "web" apps. These are the kind of apps that are intended to run on a web server and can take actions on behalf of the public at large. While they are supported, there is not yet a setup guide, so getting one running is left as an exercise for the reader. If/when a guide is assembled it will be found at L<http://redditclient.readthedocs.io/en/latest/oauth/>, with the guide for basic authentication.
 
 =head1 Constants
 
