@@ -84,13 +84,13 @@ sub build_request {
         $request->content_type('application/x-www-form-urlencoded');
         $request->content(build_query($post_data));
     } elsif ($self->{method} eq 'DELETE') {
-	$request->method('DELETE');
+		$request->method('DELETE');
     } elsif ($self->{method} eq 'PUT') {
         my $post_data = $self->{post_data} || {};
         $post_data->{modhash} = $self->{modhash} if $self->{modhash};
         $post_data->{uh}      = $self->{modhash} if $self->{modhash};
 
-	$request->method('PUT');
+		$request->method('PUT');
         $request->content_type('application/x-www-form-urlencoded');
         $request->content(build_query($post_data));
     } else {
@@ -128,7 +128,8 @@ sub send {
 			croak "Request error: HTTP ".$res->status_line .", Content: $res->{_content}";
 		} else {
 			#croak sprintf("Request error: HTTP %s last token: %s time: %s", $res->status_line, $self->{last_token}, time);
-			croak sprintf("Request error: HTTP %s", $res->status_line);
+			#croak sprintf("Request error: HTTP %s", $res->status_line);
+			die sprintf("Request error: HTTP %s\n", $res->status_line);
 		}
     }
 }
